@@ -35,6 +35,9 @@ struct Args {
 
     /// Use the MUSL build target to produce a static binary.
     musl: bool,
+
+    /// Don't actually build anything.
+    dryrun: bool,
 }
 
 enum GitHost {
@@ -60,6 +63,18 @@ impl GitHost {
 #[derive(Deserialize, Debug)]
 struct Config {
     package: Package,
+    #[serde(default)]
+    bin: Vec<Binary>,
+}
+
+impl Config {
+    /// The name of the compiled binary that should be copied to the tarball.
+    fn binary_name(&self) -> &str {
+        self.bin
+            .first()
+            .map(|bin| bin.name.as_str())
+            .unwrap_or(self.package.name.as_str())
+    }
 }
 
 #[derive(Deserialize, Debug)]
@@ -71,6 +86,52 @@ struct Package {
     homepage: String,
     repository: String,
     license: String,
+    metadata: Option<Metadata>,
+}
+
+#[derive(Deserialize, Debug)]
+struct Metadata {
+    #[serde(default)]
+    depends: Vec<String>,
+    #[serde(default)]
+    optdepends: Vec<String>,
+}
+
+impl std::fmt::Display for Metadata {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self.depends.as_slice() {
+            [middle @ .., last] => {
+                write!(f, "depends=(")?;
+                for item in middle {
+                    write!(f, "\"{}\" ", item)?;
+                }
+                if self.optdepends.is_empty().not() {
+                    writeln!(f, "\"{}\")", last)?;
+                } else {
+                    write!(f, "\"{}\")", last)?;
+                }
+            }
+            [] => {}
+        }
+
+        match self.optdepends.as_slice() {
+            [middle @ .., last] => {
+                write!(f, "optdepends=(")?;
+                for item in middle {
+                    write!(f, "\"{}\" ", item)?;
+                }
+                write!(f, "\"{}\")", last)?;
+            }
+            [] => {}
+        }
+
+        Ok(())
+    }
+}
+
+#[derive(Deserialize, Debug)]
+struct Binary {
+    name: String,
 }
 
 impl Package {
@@ -112,28 +173,31 @@ fn work(args: Args) -> Result<(), Error> {
         musl_check()?
     }
 
-    let package = cargo_config()?;
-    let license = if must_copy_license(&package.license) {
+    let config = cargo_config()?;
+    let license = if must_copy_license(&config.package.license) {
         p("LICENSE file will be installed manually.".bold().yellow());
         Some(license_file()?)
     } else {
         None
     };
-    release_build(args.musl)?;
-    tarball(args.musl, license.as_ref(), &package)?;
-    let sha256: String = sha256sum(&package)?;
 
-    // Write the PKGBUILD.
-    let file = BufWriter::new(File::create("PKGBUILD")?);
-    pkgbuild(file, &package, &sha256, license.as_ref())?;
+    if args.dryrun.not() {
+        release_build(args.musl)?;
+        tarball(args.musl, license.as_ref(), &config)?;
+        let sha256: String = sha256sum(&config.package)?;
+
+        // Write the PKGBUILD.
+        let file = BufWriter::new(File::create("PKGBUILD")?);
+        pkgbuild(file, &config, &sha256, license.as_ref())?;
+    }
 
     Ok(())
 }
 
-fn cargo_config() -> Result<Package, Error> {
+fn cargo_config() -> Result<Config, Error> {
     let content = std::fs::read_to_string("Cargo.toml")?;
     let proj: Config = toml::from_str(&content)?;
-    Ok(proj.package)
+    Ok(proj)
 }
 
 /// If a AUR package's license isn't included in `/usr/share/licenses/common/`,
@@ -160,10 +224,11 @@ fn license_file() -> Result<DirEntry, Error> {
 /// Write a legal PKGBUILD to some `Write` instance (a `File` in this case).
 fn pkgbuild<T: Write>(
     mut file: T,
-    package: &Package,
+    config: &Config,
     sha256: &str,
     license: Option<&DirEntry>,
 ) -> Result<(), Error> {
+    let package = &config.package;
     let authors = package
         .authors
         .iter()
@@ -172,7 +237,7 @@ fn pkgbuild<T: Write>(
     let source = package
         .git_host()
         .unwrap_or(GitHost::Github)
-        .source(package);
+        .source(&config.package);
 
     writeln!(file, "{}", authors)?;
     writeln!(file, "#")?;
@@ -190,6 +255,11 @@ fn pkgbuild<T: Write>(
     writeln!(file, "arch=(\"x86_64\")")?;
     writeln!(file, "provides=(\"{}\")", package.name)?;
     writeln!(file, "conflicts=(\"{}\")", package.name)?;
+
+    if let Some(metadata) = package.metadata.as_ref() {
+        writeln!(file, "{}", metadata)?;
+    }
+
     writeln!(file, "source=(\"{}\")", source)?;
     writeln!(file, "sha256sums=(\"{}\")", sha256)?;
     writeln!(file)?;
@@ -197,7 +267,7 @@ fn pkgbuild<T: Write>(
     writeln!(
         file,
         "    install -Dm755 {} -t \"$pkgdir/usr/bin\"",
-        package.name
+        config.binary_name()
     )?;
 
     if let Some(lic) = license {
@@ -229,7 +299,7 @@ fn release_build(musl: bool) -> Result<(), Error> {
     Ok(())
 }
 
-fn tarball(musl: bool, license: Option<&DirEntry>, package: &Package) -> Result<(), Error> {
+fn tarball(musl: bool, license: Option<&DirEntry>, config: &Config) -> Result<(), Error> {
     let target_dir: OsString = match std::env::var_os("CARGO_TARGET_DIR") {
         Some(p) => p,
         None => "target".into(),
@@ -241,23 +311,27 @@ fn tarball(musl: bool, license: Option<&DirEntry>, package: &Package) -> Result<
         "release"
     };
 
+    let binary_name = config.binary_name();
     let mut binary: PathBuf = target_dir.into();
     binary.push(release_dir);
-    binary.push(&package.name);
+    binary.push(binary_name);
 
     strip(&binary)?;
-    std::fs::copy(binary, &package.name)?;
+    std::fs::copy(binary, binary_name)?;
 
     // Create the tarball.
     p("Packing tarball...".bold());
     let mut command = Command::new("tar");
-    command.arg("czf").arg(package.tarball()).arg(&package.name);
+    command
+        .arg("czf")
+        .arg(config.package.tarball())
+        .arg(binary_name);
     if let Some(lic) = license {
         command.arg(lic.path());
     }
     command.status()?;
 
-    std::fs::remove_file(&package.name)?;
+    std::fs::remove_file(binary_name)?;
 
     Ok(())
 }
